@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -29,11 +29,11 @@
 #include "msg.h"
 #include "helpwin.h"
 
-#include <QtGui/QPushButton>
-#include <QtGui/QComboBox>
-#include <QtGui/QInputDialog>
-#include <QtGui/QMessageBox>
-#include <QtGui/QShortcut>
+#include <QtWidgets/QPushButton>
+#include <QtWidgets/QComboBox>
+#include <QtWidgets/QInputDialog>
+#include <QtWidgets/QMessageBox>
+#include <QtWidgets/QShortcut>
 
 #include <list>
 
@@ -244,7 +244,7 @@ void AnimSetWindow::newClicked()
    bool ok = false;
 
    QString name = QInputDialog::getText( this,
-         tr( "Misfit 3D" ),
+         tr( "New Animation" ),
          tr( "New name:" ),
          QLineEdit::Normal, QString(""), &ok );
 
@@ -282,7 +282,7 @@ void AnimSetWindow::renameClicked()
       bool ok = false;
 
       QString name = QInputDialog::getText( this,
-            tr( "Misfit 3D" ),
+            tr( "Rename Animation" ),
             tr( "New name:" ),
             QLineEdit::Normal, m_animList->item( firstSelection )->text(), &ok );
 
@@ -403,7 +403,7 @@ void AnimSetWindow::splitClicked()
          {
             bool ok = false;
             QString name = QString::fromUtf8( m_model->getAnimName( mode, t ) );
-            unsigned frame = QInputDialog::getInteger( this,
+            unsigned frame = QInputDialog::getInt( this,
                   tr("Split at frame", "Split animation frame window title" ), tr("Split", "'Split' refers to splitting an animation into two separate animations" ) 
                   + QString(" ") + name + QString(" ") + tr("at frame number", "the frame number where the second (split) animation begins" ),
                   2, 2, m_model->getAnimFrameCount( mode, t ), 1, &ok );
