@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -30,6 +30,7 @@
 #include "toolbox.h"
 #include "cmdmgr.h"
 #include "viewportsettings.h"
+#include "groupclean.h"
 #include "groupwin.h"
 #include "texwin.h"
 #include "texturecoord.h"
@@ -70,24 +71,26 @@
 #include <QtCore/QDataStream>
 #include <QtCore/QFile>
 #include <QtCore/QTimer>
-#include <QtGui/QActionGroup>
-#include <QtGui/QApplication>
+#include <QtWidgets/QActionGroup>
+#include <QtWidgets/QApplication>
 #include <QtGui/QCloseEvent>
 #include <QtGui/QContextMenuEvent>
-#include <QtGui/QDockWidget>
-#include <QtGui/QFileDialog>
+#include <QtWidgets/QDesktopWidget>
+#include <QtWidgets/QDockWidget>
+#include <QtWidgets/QFileDialog>
 #include <QtGui/QIcon>
-#include <QtGui/QLayout>
-#include <QtGui/QMenu>
-#include <QtGui/QMenuBar>
-#include <QtGui/QMessageBox>
+#include <QtWidgets/QLayout>
+#include <QtWidgets/QMenu>
+#include <QtWidgets/QMenuBar>
+#include <QtWidgets/QMessageBox>
 #include <QtGui/QPixmap>
 #include <QtGui/QResizeEvent>
-#include <QtGui/QShortcut>
-#include <QtGui/QToolBar>
-#include <QtGui/QToolButton>
-#include <QtGui/QToolTip>
-#include <QtGui/QVBoxLayout>
+#include <QtWidgets/QShortcut>
+#include <QtWidgets/QToolBar>
+#include <QtWidgets/QToolButton>
+#include <QtWidgets/QToolTip>
+#include <QtWidgets/QVBoxLayout>
+#include <QtGui/QWindow>
 
 #include "errorobj.h"
 
@@ -117,6 +120,42 @@ static ViewWindowList _winList;
 
 static bool _shuttingDown = false;
 
+CommandWidget::CommandWidget( QObject * parent, Model * model, bool * canEdit,
+      Command * cmd, int index )
+   : QObject( parent ),
+     m_model( model ),
+     m_canEdit( canEdit ),
+     m_cmd( cmd ),
+     m_index( index )
+{
+}
+
+CommandWidget::~CommandWidget()
+{
+}
+
+void CommandWidget::setModel( Model * m )
+{
+   m_model = m;
+}
+
+void CommandWidget::activateCommand( bool )
+{
+   if ( !(*m_canEdit) )
+   {
+      model_status( m_model, StatusError, STATUSTIME_LONG,
+            tr("You are in animation mode, but there are no animations").toUtf8() );
+      return;
+   }
+
+   if ( m_cmd->activated( m_index, m_model ) )
+   {
+      m_model->operationComplete( qApp->translate( "Command", (m_cmd)->getName( m_index ) ).toUtf8() );
+      DecalManager * mgr = DecalManager::getInstance();
+      mgr->modelUpdated( m_model );
+   }
+}
+
 bool ViewWindow::closeAllWindows()
 {
    bool noPrompt = true;
@@ -184,6 +223,38 @@ bool ViewWindow::closeAllWindows()
    }
 }
 
+// open model in an existing window that doesn't have a model open or open a new window.
+bool ViewWindow::openModelInEmptyWindow( const char * filename )
+{
+   QWindow *window = NULL;
+
+   std::list<ViewWindow *>::iterator it;
+
+   for ( it = _winList.begin(); it != _winList.end(); it++ )
+   {
+      if ( !(*it)->emptyWindow() ) {
+         continue;
+      }
+
+      if ( !(*it)->openModelInWindow( filename ) ) {
+         return false;
+      }
+
+      // Unminimize and move window to foreground
+      window = (*it)->windowHandle();
+      window->show();
+      window->raise();
+      window->requestActivate();
+      break;
+   }
+
+   if ( !window ) {
+      return ViewWindow::openModel( filename );
+   }
+
+   return false;
+}
+
 class MainWidget : public QWidget
 {
    public:
@@ -227,7 +298,7 @@ ViewWindow::ViewWindow( Model * model, QWidget * parent )
 
 
    setWindowIcon( QPixmap((const char **) mm3dlogo_32x32_xpm) );
-   setWindowIconText( QString("Misfit Model 3D") );
+   setWindowIconText( QString("Maverick Model 3D") );
 
    QShortcut * help = new QShortcut( QKeySequence( tr("F1", "Help Shortcut")), this );
    connect( help, SIGNAL(activated()), this, SLOT(helpNowEvent()) );
@@ -555,13 +626,14 @@ ViewWindow::ViewWindow( Model * model, QWidget * parent )
    m_geometryMenu = new QMenu( this );
    m_materialsMenu  = new QMenu( this );
 
-   m_materialsMenu->addAction( tr("Edit Groups...", "Groups|Edit Groups"), this, SLOT(groupWindowEvent()), g_keyConfig.getKey( "viewwin_groups_edit_groups" ) );
-   m_materialsMenu->addAction( tr("Edit Materials...", "Groups|Edit Materials"), this, SLOT(textureWindowEvent()), g_keyConfig.getKey( "viewwin_groups_edit_materials" ) );
-   m_materialsMenu->addAction( tr("Reload Textures", "Groups|Reload Textures"), this, SLOT(reloadTexturesEvent()), g_keyConfig.getKey( "viewwin_groups_reload_textures" ) );
+   m_materialsMenu->addAction( tr("Edit Groups...", "Materials|Edit Groups"), this, SLOT(groupWindowEvent()), g_keyConfig.getKey( "viewwin_groups_edit_groups" ) );
+   m_materialsMenu->addAction( tr("Edit Materials...", "Materials|Edit Materials"), this, SLOT(textureWindowEvent()), g_keyConfig.getKey( "viewwin_groups_edit_materials" ) );
+   m_materialsMenu->addAction( tr("Clean Up...", "Materials|Clean Up"), this, SLOT(groupCleanWindowEvent()), g_keyConfig.getKey( "viewwin_groups_cleanup" ) );
+   m_materialsMenu->addAction( tr("Reload Textures", "Materials|Reload Textures"), this, SLOT(reloadTexturesEvent()), g_keyConfig.getKey( "viewwin_groups_reload_textures" ) );
    m_materialsMenu->addSeparator();
-   m_materialsMenu->addAction( tr("Edit Projection...", "Groups|Edit Projection"), this, SLOT(projectionWindowEvent()), g_keyConfig.getKey( "viewwin_groups_edit_projection" ) );
-   m_materialsMenu->addAction( tr("Edit Texture Coordinates...", "Groups|Edit Texture Coordinates"), this, SLOT(textureCoordEvent()), g_keyConfig.getKey( "viewwin_groups_edit_texture_coordinates" ) );
-   m_materialsMenu->addAction( tr("Paint Texture...", "Groups|Paint Texture"), this, SLOT(paintTextureEvent()), g_keyConfig.getKey( "viewwin_groups_paint_texture" ) );
+   m_materialsMenu->addAction( tr("Edit Projection...", "Materials|Edit Projection"), this, SLOT(projectionWindowEvent()), g_keyConfig.getKey( "viewwin_groups_edit_projection" ) );
+   m_materialsMenu->addAction( tr("Edit Texture Coordinates...", "Materials|Edit Texture Coordinates"), this, SLOT(textureCoordEvent()), g_keyConfig.getKey( "viewwin_groups_edit_texture_coordinates" ) );
+   m_materialsMenu->addAction( tr("Paint Texture...", "Materials|Paint Texture"), this, SLOT(paintTextureEvent()), g_keyConfig.getKey( "viewwin_groups_paint_texture" ) );
 
    m_jointsMenu     = new QMenu( this );
 
@@ -581,8 +653,6 @@ ViewWindow::ViewWindow( Model * model, QWidget * parent )
 
    initializeCommands();
 
-   connect( m_geometryMenu, SIGNAL(triggered(QAction*)), this, SLOT(primitiveCommandActivated(QAction*)) );
-
    //m_scriptMenu = new QPopupMenu( this );
 
    m_animMenu = new QMenu( this );
@@ -641,12 +711,33 @@ ViewWindow::ViewWindow( Model * model, QWidget * parent )
          g_prefs.exists( "ui_viewwin_width")  ? g_prefs( "ui_viewwin_width" )  : 900,
          g_prefs.exists( "ui_viewwin_height") ? g_prefs( "ui_viewwin_height" ) : 700 );
 
+   // If the window size including the title bar etc is the area of the screen
+   // without taskbar etc then set the window to maximized.
+   QRect screenSpace = QApplication::desktop()->availableGeometry( this );
+   bool windowMaximized = false;
+   if ( g_prefs.exists( "ui_viewwin_lastframewidth") && g_prefs.exists( "ui_viewwin_lastframeheight") )
+   {
+      int frameWidth = g_prefs( "ui_viewwin_lastframewidth");
+      int frameHeight = g_prefs( "ui_viewwin_lastframeheight");
+
+      //log_debug( "Window frame size %dx%d, available %dx%d\n", frameWidth, frameHeight, screenSpace.width(), screenSpace.height() );
+
+      windowMaximized = ( frameWidth >= screenSpace.width() && frameHeight >= screenSpace.height() );
+   }
+
    setMinimumSize( 520, 520 );
 
    frameAllEvent();
 
    m_viewPanel->modelUpdatedEvent();
-   show();
+   if ( windowMaximized )
+   {
+      showMaximized();
+   }
+   else
+   {
+      show();
+   }
 
    QTimer * m_savedTimer = new QTimer();
    connect( m_savedTimer, SIGNAL(timeout()), this, SLOT(savedTimeoutCheck()) );
@@ -737,6 +828,22 @@ void ViewWindow::setModel( Model * model )
    {
       m_currentTool->setModel( m_model );
    }
+
+   for ( CommandMenuItemList::iterator it = m_primitiveCommands.begin();
+         it != m_primitiveCommands.end(); ++it )
+   {
+      (*it)->widget->setModel( m_model );
+   }
+}
+
+bool ViewWindow::emptyWindow()
+{
+   if ( m_model )
+   {
+      // if no model is loaded and default scene is unmodified
+      return ( strcmp( m_model->getFilename(), "" ) == 0 && m_model->getSaved() == true && !m_model->canRedo() );
+   }
+   return true;
 }
 
 bool ViewWindow::getSaved()
@@ -768,6 +875,11 @@ void ViewWindow::resizeEvent ( QResizeEvent * e )
    g_prefs( "ui_viewwin_height" ) = h;
 
    QMainWindow::resizeEvent(e);
+
+   g_prefs( "ui_viewwin_lastframewidth" )  = frameGeometry().width();
+   g_prefs( "ui_viewwin_lastframeheight" ) = frameGeometry().height();
+
+   //log_debug( "Resize: %dx%d, Window frame size: %dx%d\n", w, h, frameGeometry().width(), frameGeometry().height() );
 }
 
 /*
@@ -920,7 +1032,7 @@ void ViewWindow::saveModelInternal( Model * model, bool exportModel )
    {
       d.setWindowTitle( tr( "Export model" ) );
    }
-   d.selectFilter( formatsStr );
+   d.selectNameFilter( formatsStr );
    d.setFileMode( QFileDialog::AnyFile );
 
    m_abortQuit = true;
@@ -1005,7 +1117,7 @@ void ViewWindow::mergeModelsEvent()
    QFileDialog d(NULL, QString(""), dir, formatsStr + QString(";; ") + tr( "All Files (*)" ) );
 
    d.setWindowTitle( tr( "Open model file" ) );
-   d.selectFilter( formatsStr );
+   d.selectNameFilter( formatsStr );
 
    if ( QDialog::Accepted == d.exec() )
    {
@@ -1094,7 +1206,7 @@ void ViewWindow::mergeAnimationsEvent()
    QFileDialog d(NULL, QString(""), dir, formatsStr + QString(";; ") + tr( "All Files (*)" ) );
 
    d.setWindowTitle( tr( "Open model file" ) );
-   d.selectFilter( formatsStr );
+   d.selectNameFilter( formatsStr );
 
    if ( QDialog::Accepted == d.exec() )
    {
@@ -1142,7 +1254,7 @@ void ViewWindow::scriptEvent()
    QFileDialog d(NULL, QString(""), dir, formatsStr + QString(";; ") + tr( "All Files (*)" ) );
 
    d.setWindowTitle( tr( "Open model file" ) );
-   d.selectFilter( formatsStr );
+   d.selectNameFilter( formatsStr );
 
    if ( QDialog::Accepted == d.exec() )
    {
@@ -1478,32 +1590,6 @@ void ViewWindow::toolActivated( QAction * id )
    }
 }
 
-void ViewWindow::primitiveCommandActivated( QAction * id )
-{
-   if ( !m_canEdit )
-   {
-      model_status( m_model, StatusError, STATUSTIME_LONG,
-            tr("You are in animation mode, but there are no animations").toUtf8() );
-      return;
-   }
-
-   CommandMenuItemList::iterator it;
-   it = m_primitiveCommands.begin();
-   while ( it != m_primitiveCommands.end() )
-   {
-      if ( (*it)->id == id )
-      {
-         if ( ((*it)->command)->activated( (*it)->arg, m_model ) )
-         {
-            m_model->operationComplete( qApp->translate( "Command", ((*it)->command)->getName( (*it)->arg ) ).toUtf8() );
-            m_viewPanel->modelUpdatedEvent();
-         }
-         break;
-      }
-      it++;
-   }
-}
-
 void ViewWindow::scriptActivated( QAction * id )
 {
 }
@@ -1520,6 +1606,12 @@ void ViewWindow::textureWindowEvent()
    win->show();
 }
 
+void ViewWindow::groupCleanWindowEvent()
+{
+   GroupCleanWin * win = new GroupCleanWin( m_model );
+   win->show();
+}
+
 void ViewWindow::textureCoordEvent()
 {
    if ( m_model->getSelectedTriangleCount() > 0 )
@@ -1920,7 +2012,7 @@ static QString _makeToolTip( ::Tool * tool, int index )
    if ( !key.isEmpty() )
    {
       name += QString(" (");
-      name += (QString) key;
+      name += key.toString();
       name += QString(")");
    }
    return name;
@@ -1969,7 +2061,8 @@ void ViewWindow::initializeToolbox()
 
                m_toolList[m_toolCount] = tool;
                // Text below
-               m_toolButtons[ m_toolCount ] = m_toolBar->addAction( set, "Text" );
+               m_toolButtons[ m_toolCount ] = m_toolBar->addAction( set,
+                     qApp->translate( "Tool", tool->getName(t) ) );
                m_toolButtons[ m_toolCount ]->setCheckable( true );
                if ( name && name[0] )
                {
@@ -2018,7 +2111,8 @@ void ViewWindow::initializeToolbox()
 
             m_toolList[m_toolCount] = tool;
             // Text below
-            m_toolButtons[ m_toolCount ] = m_toolBar->addAction( set, "Text" );
+            m_toolButtons[ m_toolCount ] = m_toolBar->addAction( set,
+                     qApp->translate( "Tool", tool->getName(0) ) );
             m_toolButtons[ m_toolCount ]->setCheckable( true );
             if ( name && name[0] )
             {
@@ -2086,6 +2180,8 @@ void ViewWindow::initializeCommands()
                item->id = id;
                item->command = cmd;
                item->arg = t;
+               item->widget = new CommandWidget(this, m_model, &m_canEdit, cmd, t );
+               connect(id, SIGNAL(triggered(bool)), item->widget, SLOT(activateCommand(bool)));
 
                _registerKeyBinding( cmd, t, menu, id );
                m_primitiveCommands.push_back( item );
@@ -2101,6 +2197,8 @@ void ViewWindow::initializeCommands()
             item->id = id;
             item->command = cmd;
             item->arg = 0;
+            item->widget = new CommandWidget(this, m_model, &m_canEdit, cmd, 0 );
+            connect(id, SIGNAL(triggered(bool)), item->widget, SLOT(activateCommand(bool)));
 
             m_primitiveCommands.push_back( item );
          }
@@ -2114,6 +2212,8 @@ void ViewWindow::initializeCommands()
             item->id = id;
             item->command = cmd;
             item->arg = 0;
+            item->widget = new CommandWidget(this, m_model, &m_canEdit, cmd, 0 );
+            connect(id, SIGNAL(triggered(bool)), item->widget, SLOT(activateCommand(bool)));
 
             _registerKeyBinding( cmd, 0, curMenu, id );
 
@@ -2233,7 +2333,7 @@ bool ViewWindow::openModelDialog( const char * openDirectory )
    QFileDialog d(NULL, QString(""), dir, formatsStr + QString(";; ") + tr( "All Files (*)" ) );
 
    d.setWindowTitle( tr( "Open model file" ) );
-   d.selectFilter( formatsStr );
+   d.selectNameFilter( formatsStr );
 
    if ( QDialog::Accepted == d.exec() )
    {
@@ -2353,7 +2453,7 @@ bool ViewWindow::openModelDialogInWindow( const char * openDirectory )
    QFileDialog d(NULL, QString(""), dir, formatsStr + QString(";; ") + tr( "All Files (*)" ) );
 
    d.setWindowTitle( tr( "Open model file" ) );
-   d.selectFilter( formatsStr );
+   d.selectNameFilter( formatsStr );
 
    if ( QDialog::Accepted == d.exec() )
    {
@@ -2462,7 +2562,7 @@ void ViewWindow::loadDockPositions()
 
 void ViewWindow::updateCaption()
 {
-   QString caption = QString( "Misfit Model 3D: " );
+   QString caption = QString( "Maverick Model 3D: " );
    if ( m_model )
    {
       caption += m_model->getSaved() ? QString("") : QString("* ");
