@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -74,12 +74,6 @@
 #include <ctype.h>
 #include <limits.h>
 
-#ifdef WIN32
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#endif // WIN32
-
 #include <map>
 #include <vector>
 
@@ -109,17 +103,6 @@ static Cal3dFilter * s_filter = NULL;
 #define CAL3D_MAGIC_SKELETON  "CSF\0"
 #define CAL3D_MAGIC_ANIMATION "CAF\0"
 
-// FIXME centralize this
-template<typename T>
-class FunctionCaller
-{
-   public:
-      FunctionCaller( T * obj, void (T::*method)(void) ) { m_obj = obj; m_method = method; }
-      ~FunctionCaller() { (m_obj->*m_method)(); }
-   private:
-      T * m_obj;
-      void (T::*m_method)(void);
-};
 const Model::AnimationModeE MODE = Model::ANIMMODE_SKELETAL;
 
 static char * _skipSpace( char * str )
@@ -1203,7 +1186,7 @@ Model::ModelErrorE Cal3dFilter::readAnimationFile( uint8_t * buf, size_t len )
          if ( fileVersion >= CAL3D_COMP_ANIM_VERSION )
          {
             int flags = readBInt32();
-            compressed = ( flags & 1 != 0 );
+            compressed = ( (flags & 1) != 0 );
          }
          //name += ".";
          //name += m_modelPartExt;
@@ -1253,7 +1236,7 @@ Model::ModelErrorE Cal3dFilter::readFileToBuffer( const char * filename, uint8_t
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    DataSource * src = openInput( filename, err );
-   FunctionCaller<DataSource> fc( src, &DataSource::close );
+   SourceCloser fc( src );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -1993,7 +1976,7 @@ Model::ModelErrorE Cal3dFilter::writeCal3dFile( const char * filename, Model * m
 {
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_dst = openOutput( filename, err );
-   FunctionCaller<DataDest> fc( m_dst, &DataDest::close );
+   DestCloser fc( m_dst );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -2021,8 +2004,8 @@ Model::ModelErrorE Cal3dFilter::writeCal3dFile( const char * filename, Model * m
 
    m_dst->writeString( "#\n# cal3d model configuration file\n#\n" );
 
-   m_dst->writeString( "# File written by Misfit Model 3D\n" );
-   m_dst->writeString( "# http://www.misfitcode.com/misfitmodel3d/\n\n" );
+   m_dst->writeString( "# File written by Maverick Model 3D\n" );
+   m_dst->writeString( "# https://clover.com/mm3d\n\n" );
 
    m_dst->writeString( "[model]\n" );
 
@@ -2186,7 +2169,7 @@ Model::ModelErrorE Cal3dFilter::writeXMaterialFile( const char * filename, Model
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_dst = openOutput( filename, err );
-   FunctionCaller<DataDest> fc( m_dst, &DataDest::close );
+   DestCloser fc( m_dst );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -2253,7 +2236,7 @@ Model::ModelErrorE Cal3dFilter::writeSkeletonFile( const char * filename, Model
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_dst = openOutput( filename, err );
-   FunctionCaller<DataDest> fc( m_dst, &DataDest::close );
+   DestCloser fc( m_dst );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -2296,7 +2279,7 @@ Model::ModelErrorE Cal3dFilter::writeMeshListFile( const char * filename, Model
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_dst = openOutput( filename, err );
-   FunctionCaller<DataDest> fc( m_dst, &DataDest::close );
+   DestCloser fc( m_dst );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -2332,7 +2315,7 @@ Model::ModelErrorE Cal3dFilter::writeMaterialFile( const char * filename, Model
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_dst = openOutput( filename, err );
-   FunctionCaller<DataDest> fc( m_dst, &DataDest::close );
+   DestCloser fc( m_dst );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -2393,7 +2376,7 @@ Model::ModelErrorE Cal3dFilter::writeAnimationFile( const char * filename, Model
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_dst = openOutput( filename, err );
-   FunctionCaller<DataDest> fc( m_dst, &DataDest::close );
+   DestCloser fc( m_dst );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -2750,7 +2733,7 @@ int Cal3dFilter::findAnimation( const std::string& animName )
 // Plugin functions
 //------------------------------------------------------------------
 
-extern "C" bool plugin_init()
+PLUGIN_API bool plugin_init()
 {
    if ( s_filter == NULL )
    {
@@ -2764,19 +2747,24 @@ extern "C" bool plugin_init()
 
 // The filter manager will delete our registered filter.
 // We have no other cleanup to do
-extern "C" bool plugin_uninit()
+PLUGIN_API bool plugin_uninit()
 {
    s_filter = NULL; // FilterManager deletes filters
    log_debug( "CAL3D model filter plugin uninitialized\n" );
    return true;
 }
 
-extern "C" const char * plugin_version()
+PLUGIN_API const char * plugin_mm3d_version()
+{
+   return VERSION_STRING;
+}
+
+PLUGIN_API const char * plugin_version()
 {
    return "0.1.0";
 }
 
-extern "C" const char * plugin_desc()
+PLUGIN_API const char * plugin_desc()
 {
    return "CAL3D model filter";
 }
