@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -48,18 +48,6 @@ using std::string;
 static DxfFilter * s_filter = NULL;
 #endif // PLUGIN
 
-// FIXME centralize this
-template<typename T>
-class FunctionCaller
-{
-   public:
-      FunctionCaller( T * obj, void (T::*method)(void) ) { m_obj = obj; m_method = method; }
-      ~FunctionCaller() { (m_obj->*m_method)(); }
-   private:
-      T * m_obj;
-      void (T::*m_method)(void);
-};
-
 static bool _float_equiv( float rhs, float lhs )
 {
    if ( fabs( rhs - lhs ) < 0.0001f )
@@ -369,7 +357,7 @@ Model::ModelErrorE DxfFilter::readFile( Model * model, const char * const filena
 {
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_src = openInput( filename, err );
-   FunctionCaller<DataSource> fc( m_src, &DataSource::close );
+   SourceCloser fc( m_src );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -425,7 +413,7 @@ Model::ModelErrorE DxfFilter::writeFile( Model * model, const char * const filen
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_dst = openOutput( filename, err );
-   FunctionCaller<DataDest> fc( m_dst, &DataDest::close );
+   DestCloser fc( m_dst );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -728,7 +716,7 @@ bool DxfFilter::read3dface( const char * line )
                     if ( m_lastCode == 33 )
                     {
                         // add polygon
-                        int vert[3];
+                        int vert[4];
                         int tri;
 
                         // Exchange Y and Z coordinates
@@ -1213,7 +1201,7 @@ int DxfFilter::getMaterialColor( unsigned int material )
 // Plugin functions
 //------------------------------------------------------------------
 
-extern "C" bool plugin_init()
+PLUGIN_API bool plugin_init()
 {
    if ( s_filter == NULL )
    {
@@ -1227,19 +1215,24 @@ extern "C" bool plugin_init()
 
 // The filter manager will delete our registered filter.
 // We have no other cleanup to do
-extern "C" bool plugin_uninit()
+PLUGIN_API bool plugin_uninit()
 {
    s_filter = NULL; // FilterManager deletes filters
    log_debug( "DXF model filter plugin uninitialized\n" );
    return true;
 }
 
-extern "C" const char * plugin_version()
+PLUGIN_API const char * plugin_mm3d_version()
+{
+   return VERSION_STRING;
+}
+
+PLUGIN_API const char * plugin_version()
 {
    return "0.1.0";
 }
 
-extern "C" const char * plugin_desc()
+PLUGIN_API const char * plugin_desc()
 {
    return "DXF model filter";
 }
