@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -26,24 +26,33 @@
 #include <math.h>
 #include <string.h>
 
+static const double EQ_TOLERANCE = 0.00001;
+
 Matrix::Matrix()
 {
    loadIdentity();
 }
 
-Matrix::~Matrix()
-{
-}
-
 void Matrix::loadIdentity()
 {
-   for ( int r = 0; r < 4; r++ )
+   memset( m_val, 0, sizeof(m_val) );
+   m_val[0] = 1.0;
+   m_val[5] = 1.0;
+   m_val[10] = 1.0;
+   m_val[15] = 1.0;
+}
+
+bool Matrix::isIdentity() const
+{
+   for ( int c = 0; c < 4; ++c )
    {
-      for ( int c = 0; c < 4; c++ )
+      for ( int r = 0; r < 4; ++r )
       {
-         m_val[ (r<<2) + c ] = (c==r) ? 1.0 : 0.0;
+         if ( !float_equiv(get(r, c), (r == c) ? 1.0 : 0.0 ) )
+            return false;
       }
    }
+   return true;
 }
 
 void Matrix::show() const
@@ -58,14 +67,45 @@ void Matrix::show() const
    }
 }
 
-void Matrix::set( int r, int c, double val )
+bool Matrix::operator==( const Matrix & rhs ) const
 {
-   m_val[ (r<<2) + c ] = val;
+   for ( int c = 0; c < 4; ++c )
+   {
+      for ( int r = 0; r < 4; ++r )
+      {
+         if ( !float_equiv(get(r, c), rhs.get(r, c) ) )
+            return false;
+      }
+   }
+   return true;
 }
 
-double Matrix::get( int r, int c ) const
+bool Matrix::equiv( const Matrix & rhs, double tolerance ) const
 {
-   return m_val[ (r<<2) + c ];
+   Vector lright( 2, 0, 0 );
+   Vector lup( 0, 2, 0 );
+   Vector lfront( 0, 0, 2 );
+
+   Vector rright( 2, 0, 0 );
+   Vector rup( 0, 2, 0 );
+   Vector rfront( 0, 0, 2 );
+
+   this->apply( lright );
+   this->apply( lup );
+   this->apply( lfront );
+
+   rhs.apply( rright );
+   rhs.apply( rup );
+   rhs.apply( rfront );
+
+   if ( (lright - rright).mag() > tolerance )
+      return false;
+   if ( (lup - rup).mag() > tolerance )
+      return false;
+   if ( (lfront - rfront).mag() > tolerance )
+      return false;
+
+   return true;
 }
 
 void Matrix::setTranslation( const double * vector )
@@ -325,7 +365,7 @@ void Matrix::setRotationQuaternion( const Quaternion & quat )
    m_val[15] = 1;
 }
 
-void Matrix::getRotationQuaternion( Quaternion & quat )
+void Matrix::getRotationQuaternion( Quaternion & quat ) const
 {
    double s;
 
@@ -566,9 +606,10 @@ float Matrix::getDeterminant() const
 {
    float result = 0;
 
+   Matrix msub3;
    for ( int n = 0, i = 1; n < 4; n++, i *= -1 )
    {
-      Matrix msub3 = getSubMatrix( 0, n );
+      getSubMatrix( msub3, 0, n );
 
       float det = msub3.getDeterminant3();
       result += m_val[n] * det * i;
@@ -577,9 +618,8 @@ float Matrix::getDeterminant() const
    return( result );
 }
 
-Matrix Matrix::getSubMatrix( int i, int j ) const
+void Matrix::getSubMatrix( Matrix & ret, int i, int j ) const
 {
-   Matrix ret;
    for( int di = 0; di < 3; di ++ ) 
    {
       for( int dj = 0; dj < 3; dj ++ ) 
@@ -587,11 +627,9 @@ Matrix Matrix::getSubMatrix( int i, int j ) const
          int si = di + ( ( di >= i ) ? 1 : 0 );
          int sj = dj + ( ( dj >= j ) ? 1 : 0 );
 
-         ret.set( di, dj, m_val[si * 4 + sj] );
+         ret.set( di, dj, m_val[(si<<2) + sj] );
       }
    }
-
-   return ret;
 }
 
 Matrix Matrix::getInverse() const
@@ -605,12 +643,13 @@ Matrix Matrix::getInverse() const
       return mr;
    }
 
+   Matrix mtemp;
    for ( int i = 0; i < 4; i++ )
    {
       for ( int j = 0; j < 4; j++ )
       {
          int sign = 1 - ( (i +j) % 2 ) * 2;
-         Matrix mtemp = getSubMatrix( i, j );
+         getSubMatrix( mtemp, i, j );
          mr.set( j, i, ( mtemp.getDeterminant3() * sign ) / mdet);
       }
    }
@@ -822,7 +861,7 @@ void Vector::scale3( double scale )
    }
 }
 
-double Vector::mag()
+double Vector::mag() const
 {
    return sqrt(   m_val[0]*m_val[0]
                 + m_val[1]*m_val[1]
@@ -830,7 +869,7 @@ double Vector::mag()
                 + m_val[3]*m_val[3] );
 }
 
-double Vector::mag3()
+double Vector::mag3() const
 {
    return sqrt(   m_val[0]*m_val[0]
                 + m_val[1]*m_val[1]
@@ -855,13 +894,21 @@ void Vector::normalize3()
    }
 }
 
-double Vector::dot3( const Vector & rhs )
+double Vector::dot3( const Vector & rhs ) const
 {
    return(   m_val[0] * rhs.m_val[0]
            + m_val[1] * rhs.m_val[1]
            + m_val[2] * rhs.m_val[2] );
 }
 
+double Vector::dot4( const Vector & rhs ) const
+{
+   return(   m_val[0] * rhs.m_val[0]
+           + m_val[1] * rhs.m_val[1]
+           + m_val[2] * rhs.m_val[2]
+           + m_val[3] * rhs.m_val[3] );
+}
+
 Vector Vector::cross3( const Vector & rhs ) const
 {
    Vector rval;
@@ -901,6 +948,11 @@ Vector Vector::operator-=( const Vector & rhs )
    return *this;
 }
 
+bool Vector::operator==( const Vector & rhs ) const
+{
+   return floatCompareVector(this->getVector(), rhs.getVector(), 4);
+}
+
 Quaternion::Quaternion( const double * val )
 {
    if ( val )
@@ -1061,11 +1113,12 @@ void Quaternion::setRotationToPoint( const double & faceX, const double & faceY,
    setRotationOnAxis( normal.getVector(), angle );
 }
 
-void Quaternion::getRotationOnAxis( double * axis, double & radians )
+void Quaternion::getRotationOnAxis( double * axis, double & radians ) const
 {
-   normalize();
+   Quaternion q(m_val);
+   q.normalize();
 
-   double cos_a = m_val[3];
+   double cos_a = q.m_val[3];
    radians = acos( cos_a ) * 2;
 
    double sin_a = sqrt( 1.0 - cos_a * cos_a );
@@ -1074,9 +1127,9 @@ void Quaternion::getRotationOnAxis( double * axis, double & radians )
       sin_a = 1.0;
    }
 
-   axis[0] = m_val[0] / sin_a;
-   axis[1] = m_val[1] / sin_a;
-   axis[2] = m_val[2] / sin_a;
+   axis[0] = q.m_val[0] / sin_a;
+   axis[1] = q.m_val[1] / sin_a;
+   axis[2] = q.m_val[2] / sin_a;
 }
 
 void Quaternion::normalize()
