@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -45,22 +45,10 @@
 using std::list;
 using std::string;
 
-template<typename T>
-class FunctionCaller
-{
-   public:
-      FunctionCaller( T * obj, void (T::*method)(void) ) { m_obj = obj; m_method = method; }
-      ~FunctionCaller() { (m_obj->*m_method)(); }
-   private:
-      T * m_obj;
-      void (T::*m_method)(void);
-};
-
-
 const char     MisfitFilter::MAGIC[] = "MISFIT3D";
 
 const uint8_t  MisfitFilter::WRITE_VERSION_MAJOR = 0x01;
-const uint8_t  MisfitFilter::WRITE_VERSION_MINOR = 0x05;  // FIXME bump version
+const uint8_t  MisfitFilter::WRITE_VERSION_MINOR = 0x07;
 
 const uint16_t MisfitFilter::OFFSET_TYPE_MASK  = 0x3fff;
 const uint16_t MisfitFilter::OFFSET_UNI_MASK   = 0x8000;
@@ -357,6 +345,16 @@ typedef enum _MisfitFlags_e
    MF_MAT_CLAMP_T = 32
 } MisfitFlagsE;
 
+typedef enum _MisfitFrameAnimFlags_e
+{
+   MFAF_ANIM_LOOP = 0x0001
+} MisfitFrameAnimFlagsE;
+
+typedef enum _MisfitSkelAnimFlags_e
+{
+   MSAF_ANIM_LOOP = 0x0001
+} MisfitSkelAnimFlagsE;
+
 static const uint16_t _misfitOffsetTypes[MDT_MAX]  = {
 
    // Offset A types
@@ -688,7 +686,7 @@ Model::ModelErrorE MisfitFilter::readFile( Model * model, const char * const fil
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_src = openInput( filename, err );
-   FunctionCaller<DataSource> fc( m_src, &DataSource::close );
+   SourceCloser fc( m_src );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -1801,6 +1799,7 @@ Model::ModelErrorE MisfitFilter::readFile( Model * model, const char * const fil
          unsigned anim = model->addAnimation( Model::ANIMMODE_SKELETAL, name );
          model->setAnimFPS( Model::ANIMMODE_SKELETAL, anim, fps );
          model->setAnimFrameCount( Model::ANIMMODE_SKELETAL, anim, frameCount );
+         model->setAnimLooping( Model::ANIMMODE_SKELETAL, anim, (flags & MSAF_ANIM_LOOP) != 0 );
 
          for ( unsigned f = 0; f < frameCount; f++ )
          {
@@ -1886,6 +1885,7 @@ Model::ModelErrorE MisfitFilter::readFile( Model * model, const char * const fil
          unsigned anim = model->addAnimation( Model::ANIMMODE_FRAME, name );
          model->setAnimFPS( Model::ANIMMODE_FRAME, anim, fps );
          model->setAnimFrameCount( Model::ANIMMODE_FRAME, anim, frameCount );
+         model->setAnimLooping( Model::ANIMMODE_FRAME, anim, (flags & MFAF_ANIM_LOOP) != 0 );
 
          for ( unsigned f = 0; f < frameCount; f++ )
          {
@@ -2047,9 +2047,9 @@ Model::ModelErrorE MisfitFilter::readFile( Model * model, const char * const fil
          if ( modelJoints[j]->m_parent >= (signed) jcount )
          {
             log_warning( "Joint %d has bad parent joint, checking endianness\n", j );
-            if ( htonl( modelJoints[j]->m_parent ) < jcount )
+            if ( htob_u32( modelJoints[j]->m_parent ) < jcount )
             {
-               modelJoints[j]->m_parent = htonl( modelJoints[j]->m_parent );
+               modelJoints[j]->m_parent = htob_u32( modelJoints[j]->m_parent );
             }
             else if ( htol_u32( modelJoints[j]->m_parent ) < jcount )
             {
@@ -2091,7 +2091,7 @@ Model::ModelErrorE MisfitFilter::writeFile( Model * model, const char * const fi
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_dst = openOutput( filename, err );
-   FunctionCaller<DataDest> fc( m_dst, &DataDest::close );
+   DestCloser fc( m_dst );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -2982,7 +2982,11 @@ Model::ModelErrorE MisfitFilter::writeFile( Model * model, const char * const fi
          uint16_t  flags = 0x0000;
          float32_t fps = sa->m_fps;
 
-         animSize = animSize;
+         if (sa->m_loop)
+         {
+            flags |= MSAF_ANIM_LOOP;
+         }
+
          m_dst->write( animSize );
          m_dst->write( flags );
          m_dst->writeBytes( (const uint8_t *) sa->m_name.c_str(), sa->m_name.length() + 1 );
@@ -3064,6 +3068,11 @@ Model::ModelErrorE MisfitFilter::writeFile( Model * model, const char * const fi
          uint16_t  flags = 0x0000;
          float32_t fps = fa->m_fps;
 
+         if (fa->m_loop)
+         {
+            flags |= MFAF_ANIM_LOOP;
+         }
+
          m_dst->write( animSize );
          m_dst->write( flags );
          m_dst->writeBytes( (const uint8_t *) fa->m_name.c_str(), fa->m_name.length() + 1 );
